<?php

class Mlumen {
	/**
	 * @var Mlumen_Options
	 */
	private $options;

	/**
	 * @var Mlumen_Api_Blog
	 */
	private $api_blog;

	/**
	 * @var Mlumen_Api_Descriptor
	 */
	private $api_descriptor;

	/**
	 * Check version, add action and init
	 */
	public function __construct() {
		// check version PHP and Wordpress
		if ( ! $this->check_minimal_versions() ) {
			add_action( 'plugins_loaded', array( $this, 'load_text' ) );
			add_action( 'plugins_loaded', array( $this, 'install_info' ) );

			return;
		}

		$this->load_css();
		$this->load_js();

		$this->options        = new Mlumen_Options();
		$this->api_blog       = new Mlumen_Api_Blog( MLUMEN_API_BLOG );
		$this->api_descriptor = new Mlumen_Api_Descriptor( MLUMEN_API_DESCRIPTOR, $this->options );

		register_activation_hook( MLUMEN_PLUGIN_FILE, array( $this, 'activate' ) );
		register_deactivation_hook( MLUMEN_PLUGIN_FILE, array( $this, 'deactivate' ) );

		add_filter( 'plugin_action_links', array( $this, 'plugin_action_links' ), 10, 2 );
		add_action( 'plugins_loaded', array( $this, 'load_text' ) );
		add_action( 'plugins_loaded', array( $this, 'init' ) );
	}

	/**
	 * Check minimal version PHP and Wordpress
	 *
	 * @return bool
	 */
	public function check_minimal_versions() {
		global $wp_version;

		if ( version_compare( $wp_version, MLUMEN_MINIMUM_WP_VERSION, '<' ) ) {
			return false;
		}

		if ( version_compare( PHP_VERSION, MLUMEN_MINIMUM_PHP_VERSION, '<' ) ) {
			return false;
		}

		return true;
	}

	/**
	 * The code that runs during plugin activation
	 */
	public function activate() {

	}

	/**
	 * The code that runs during plugin deactivation
	 */
	public function deactivate() {

	}

	/**
	 * Plugin action, add links "Settings"
	 *
	 * @return array
	 */
	public function plugin_action_links( $links, $file ) {
		if ( ! current_user_can( 'manage_options' ) ) {
			return $links;
		}

		if ( $file === 'mlumen/mlumen.php' ) {
			$settings_link = '<a href="' . esc_url( get_admin_url() ) . 'options-general.php?page=mlumen">' . __( 'Ustawienia', 'mlumen' ) . '</a>';
			array_unshift( $links, $settings_link );
		}

		return $links;
	}

	/**
	 * Init
	 *
	 */
	public function init() {
		$this->register_meta_box();

		if ( current_user_can( 'edit_posts' ) ) {
			$this->descriptor_single_post();
			$this->descriptor_bulk_posts();
		}

		if ( current_user_can( 'manage_options' ) ) {
			$this->install_info();
			$this->option_page();
		}
	}

	/**
	 * Localization
	 */
	public function load_text() {
		load_plugin_textdomain( 'mlumen', false, MLUMEN_PLUGIN_DIRNAME . '/languages' );
	}

	/**
	 * Load css
	 */
	public function load_css() {
		add_action( 'admin_enqueue_scripts', function () {
			wp_enqueue_style( 'thickbox' );
			wp_enqueue_style( 'lumen_admin_css', MLUMEN_PLUGIN_URL . 'admin/css/style.css' );
		} );
	}

	/**
	 * Load javascript
	 */
	public function load_js() {
		add_action( 'admin_enqueue_scripts', function () {

			wp_enqueue_script( 'thickbox' );
			wp_enqueue_script( 'lumen_admin_js', MLUMEN_PLUGIN_URL . 'admin/js/classic-editor-tags.min.js', array( 'jquery' ) );
			wp_enqueue_script( 'lumen_descriptor_bulk', MLUMEN_PLUGIN_URL . 'admin/js/descriptor-bulk.min.js', array( 'jquery' ) );
			wp_enqueue_script( 'lumen_admin_options', MLUMEN_PLUGIN_URL . 'admin/js/options.min.js', array( 'jquery' ) );

			wp_localize_script(
				'lumen_admin_js',
				'mlumen_options',
				array(
					'ajaxurl'               => admin_url( 'admin-ajax.php' ),
					'nonce'                 => wp_create_nonce( 'mlumen' ),
					'taxonomy_prompts_type' => (int) $this->options->options['taxonomy_prompts_type'],
					'taxonomy_prompts'      => $this->options->get_taxonomy_prompts()
				)
			);
		} );
	}

	/**
	 * Install info
	 */
	public function install_info() {
		new Mlumen_Install_Info( $this->options );
	}

	/**
	 * Option page
	 */
	public function option_page() {
		new Mlumen_Option_Page( $this->api_blog, $this->options );
	}

	/**
	 * Register meta box
	 */
	public function register_meta_box() {
		$descriptor_single_post = new MLumen_Descriptor_Single_Post( $this->options, $this->api_descriptor );
		$descriptor_single_post->register_meta_box();
	}

	/**
	 * Descriptor single post
	 */
	public function descriptor_single_post() {
		$descriptor_single_post = new MLumen_Descriptor_Single_Post( $this->options, $this->api_descriptor );

		add_action( 'init', array( $descriptor_single_post, 'gutenberg_init' ) );
		add_action( 'init', array( $descriptor_single_post, 'register_taxonomy_mlumen' ) );
		add_action( 'save_post', array( $descriptor_single_post, 'post_auto_save' ), 10, 2 );

		add_action( 'wp_ajax_singledescriptor', array( $this, 'ajax_descriptor' ) );
	}

	/**
	 * Descriptor bulk post
	 */
	public function descriptor_bulk_posts() {
		$descriptor_bulk_posts = new MLumen_Descriptor_Bulk_Posts( $this->options, $this->api_descriptor );
		$descriptor_bulk_posts->add_bulk_actions();
	}

	/**
	 * Ajax descriptor
	 */
	public function ajax_descriptor() {
		$this->api_descriptor->get_ajax_descriptors();
	}

}